@@ -1,7 +1,7 @@
-# Copyright 1999-2010 Gentoo Foundation
+# Copyright 1999-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
-import platform
+import io
 import stat
 import textwrap
 from _emerge.SpawnProcess import SpawnProcess
@@ -13,17 +13,15 @@ from portage.localization import _
 from portage.package.ebuild._ipc.ExitCommand import ExitCommand
 from portage.package.ebuild._ipc.QueryCommand import QueryCommand
 from portage import os
-from portage import StringIO
-from portage import _encodings
-from portage import _unicode_decode
 from portage.util._pty import _create_pty_or_pipe
 from portage.util import apply_secpass_permissions
 
 class AbstractEbuildProcess(SpawnProcess):
 
 	__slots__ = ('phase', 'settings',) + \
-		('_build_dir', '_ipc_daemon', '_exit_command',)
+		('_build_dir', '_ipc_daemon', '_exit_command', '_exit_timeout_id')
 	_phases_without_builddir = ('clean', 'cleanrm', 'depend', 'help',)
+	_phases_interactive_whitelist = ('config',)
 
 	# Number of milliseconds to allow natural exit of the ebuild
 	# process after it has called the exit command via IPC. It
@@ -57,7 +55,7 @@ class AbstractEbuildProcess(SpawnProcess):
 			"since PORTAGE_BUILDIR does not exist: '%s'") % \
 			(self.phase, self.settings['PORTAGE_BUILDDIR'])
 			self._eerror(textwrap.wrap(msg, 72))
-			self._set_returncode((self.pid, 1))
+			self._set_returncode((self.pid, 1 << 8))
 			self.wait()
 			return
 
@@ -95,7 +93,20 @@ class AbstractEbuildProcess(SpawnProcess):
 			else:
 				self.settings.pop('PORTAGE_EBUILD_EXIT_FILE', None)
 
-		SpawnProcess._start(self)
+		if self.fd_pipes is None:
+			self.fd_pipes = {}
+		null_fd = None
+		if 0 not in self.fd_pipes and \
+			self.phase not in self._phases_interactive_whitelist and \
+			"interactive" not in self.settings.get("PROPERTIES", "").split():
+			null_fd = os.open('/dev/null', os.O_RDONLY)
+			self.fd_pipes[0] = null_fd
+
+		try:
+			SpawnProcess._start(self)
+		finally:
+			if null_fd is not None:
+				os.close(null_fd)
 
 	def _init_ipc_fifos(self):
 
@@ -132,9 +143,14 @@ class AbstractEbuildProcess(SpawnProcess):
 		self._exit_command.reply_hook = self._exit_command_callback
 		query_command = QueryCommand(self.settings, self.phase)
 		commands = {
-			'best_version' : query_command,
-			'exit'         : self._exit_command,
-			'has_version'  : query_command,
+			'available_eclasses'  : query_command,
+			'best_version'        : query_command,
+			'eclass_path'         : query_command,
+			'exit'                : self._exit_command,
+			'has_version'         : query_command,
+			'license_path'        : query_command,
+			'master_repositories' : query_command,
+			'repository_path'     : query_command,
 		}
 		input_fifo, output_fifo = self._init_ipc_fifos()
 		self._ipc_daemon = EbuildIpcDaemon(commands=commands,
@@ -146,13 +162,29 @@ class AbstractEbuildProcess(SpawnProcess):
 	def _exit_command_callback(self):
 		if self._registered:
 			# Let the process exit naturally, if possible.
-			self.scheduler.schedule(self._reg_id, timeout=self._exit_timeout)
-			if self._registered:
-				# If it doesn't exit naturally in a reasonable amount
-				# of time, kill it (solves bug #278895). We try to avoid
-				# this when possible since it makes sandbox complain about
-				# being killed by a signal.
-				self.cancel()
+			self._exit_timeout_id = \
+				self.scheduler.timeout_add(self._exit_timeout,
+				self._exit_command_timeout_cb)
+
+	def _exit_command_timeout_cb(self):
+		if self._registered:
+			# If it doesn't exit naturally in a reasonable amount
+			# of time, kill it (solves bug #278895). We try to avoid
+			# this when possible since it makes sandbox complain about
+			# being killed by a signal.
+			self.cancel()
+			self._exit_timeout_id = \
+				self.scheduler.timeout_add(self._cancel_timeout,
+					self._cancel_timeout_cb)
+		else:
+			self._exit_timeout_id = None
+
+		return False # only run once
+
+	def _cancel_timeout_cb(self):
+		self._exit_timeout_id = None
+		self.wait()
+		return False # only run once
 
 	def _orphan_process_warn(self):
 		phase = self.phase
@@ -181,6 +213,11 @@ class AbstractEbuildProcess(SpawnProcess):
 		return not ('sesandbox' in self.settings.features \
 			and self.settings.selinux_enabled()) or os.isatty(slave_fd)
 
+	def _killed_by_signal(self, signum):
+		msg = _("The ebuild phase '%s' has been "
+		"killed by signal %s.") % (self.phase, signum)
+		self._eerror(textwrap.wrap(msg, 72))
+
 	def _unexpected_exit(self):
 
 		phase = self.phase
@@ -211,7 +248,7 @@ class AbstractEbuildProcess(SpawnProcess):
 		self._elog('eerror', lines)
 
 	def _elog(self, elog_funcname, lines):
-		out = StringIO()
+		out = io.StringIO()
 		phase = self.phase
 		elog_func = getattr(elog_messages, elog_funcname)
 		global_havecolor = portage.output.havecolor
@@ -222,27 +259,47 @@ class AbstractEbuildProcess(SpawnProcess):
 				elog_func(line, phase=phase, key=self.settings.mycpv, out=out)
 		finally:
 			portage.output.havecolor = global_havecolor
-		msg = _unicode_decode(out.getvalue(),
-			encoding=_encodings['content'], errors='replace')
+		msg = out.getvalue()
 		if msg:
-			self.scheduler.output(msg,
-				log_path=self.settings.get("PORTAGE_LOG_FILE"))
+			log_path = None
+			if self.settings.get("PORTAGE_BACKGROUND") != "subprocess":
+				log_path = self.settings.get("PORTAGE_LOG_FILE")
+			self.scheduler.output(msg, log_path=log_path)
+
+	def _log_poll_exception(self, event):
+		self._elog("eerror",
+			["%s received strange poll event: %s\n" % \
+			(self.__class__.__name__, event,)])
 
 	def _set_returncode(self, wait_retval):
 		SpawnProcess._set_returncode(self, wait_retval)
 
+		if self._exit_timeout_id is not None:
+			self.scheduler.source_remove(self._exit_timeout_id)
+			self._exit_timeout_id = None
+
 		if self._ipc_daemon is not None:
 			self._ipc_daemon.cancel()
 			if self._exit_command.exitcode is not None:
 				self.returncode = self._exit_command.exitcode
 			else:
-				self.returncode = 1
-				self._unexpected_exit()
+				if self.returncode < 0:
+					if not self.cancelled:
+						self._killed_by_signal(-self.returncode)
+				else:
+					self.returncode = 1
+					if not self.cancelled:
+						self._unexpected_exit()
 			if self._build_dir is not None:
 				self._build_dir.unlock()
 				self._build_dir = None
-		else:
+		elif not self.cancelled:
 			exit_file = self.settings.get('PORTAGE_EBUILD_EXIT_FILE')
 			if exit_file and not os.path.exists(exit_file):
-				self.returncode = 1
-				self._unexpected_exit()
+				if self.returncode < 0:
+					if not self.cancelled:
+						self._killed_by_signal(-self.returncode)
+				else:
+					self.returncode = 1
+					if not self.cancelled:
+						self._unexpected_exit()
