@@ -1,10 +1,8 @@
-# Copyright 1999-2011 Gentoo Foundation
+# Copyright 1999-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from portage import os
 from _emerge.AbstractPollTask import AbstractPollTask
-from _emerge.PollConstants import PollConstants
-import errno
 import fcntl
 
 class PipeReader(AbstractPollTask):
@@ -17,33 +15,33 @@ class PipeReader(AbstractPollTask):
 	"""
 
 	__slots__ = ("input_files",) + \
-		("_read_data", "_reg_ids")
+		("_read_data", "_reg_ids", "_use_array")
 
 	def _start(self):
 		self._reg_ids = set()
 		self._read_data = []
-		for k, f in self.input_files.items():
+
+		if self._use_array:
+			output_handler = self._array_output_handler
+		else:
+			output_handler = self._output_handler
+
+		for f in self.input_files.values():
 			fcntl.fcntl(f.fileno(), fcntl.F_SETFL,
 				fcntl.fcntl(f.fileno(), fcntl.F_GETFL) | os.O_NONBLOCK)
-			self._reg_ids.add(self.scheduler.register(f.fileno(),
-				self._registered_events, self._output_handler))
+			self._reg_ids.add(self.scheduler.io_add_watch(f.fileno(),
+				self._registered_events, output_handler))
 		self._registered = True
 
-	def isAlive(self):
-		return self._registered
-
 	def _cancel(self):
+		self._unregister()
 		if self.returncode is None:
-			self.returncode = 1
+			self.returncode = self._cancelled_returncode
 
 	def _wait(self):
 		if self.returncode is not None:
 			return self.returncode
-
-		if self._registered:
-			self.scheduler.schedule(self._reg_ids)
-			self._unregister()
-
+		self._wait_loop()
 		self.returncode = os.EX_OK
 		return self.returncode
 
@@ -57,26 +55,42 @@ class PipeReader(AbstractPollTask):
 
 	def _output_handler(self, fd, event):
 
-		if event & PollConstants.POLLIN:
-
-			while True:
-				data = None
-				try:
-					data = os.read(fd, self._bufsize)
-				except OSError as e:
-					if e.errno not in (errno.EAGAIN,):
-						raise
-					break
-				else:
-					if data:
-						self._read_data.append(data)
-					else:
-						self._unregister()
-						self.wait()
-						break
+		while True:
+			data = self._read_buf(fd, event)
+			if data is None:
+				break
+			if data:
+				self._read_data.append(data)
+			else:
+				self._unregister()
+				self.wait()
+				break
 
 		self._unregister_if_appropriate(event)
 
+		return True
+
+	def _array_output_handler(self, fd, event):
+
+		for f in self.input_files.values():
+			if f.fileno() == fd:
+				break
+
+		while True:
+			data = self._read_array(f, event)
+			if data is None:
+				break
+			if data:
+				self._read_data.append(data)
+			else:
+				self._unregister()
+				self.wait()
+				break
+
+		self._unregister_if_appropriate(event)
+
+		return True
+
 	def _unregister(self):
 		"""
 		Unregister from the scheduler and close open files.
@@ -86,7 +100,7 @@ class PipeReader(AbstractPollTask):
 
 		if self._reg_ids is not None:
 			for reg_id in self._reg_ids:
-				self.scheduler.unregister(reg_id)
+				self.scheduler.source_remove(reg_id)
 			self._reg_ids = None
 
 		if self.input_files is not None:
