@@ -1,8 +1,8 @@
 # portage.py -- core Portage functionality
-# Copyright 1998-2011 Gentoo Foundation
+# Copyright 1998-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
-VERSION="2.1.10.41"
+VERSION="2.1.11.31"
 
 # ===========================================================================
 # START OF IMPORTS -- START OF IMPORTS -- START OF IMPORTS -- START OF IMPORT
@@ -16,14 +16,6 @@ try:
 		errno.ESTALE = -1
 	import re
 	import types
-
-	# Try the commands module first, since this allows us to eliminate
-	# the subprocess module from the baseline imports under python2.
-	try:
-		from commands import getstatusoutput as subprocess_getstatusoutput
-	except ImportError:
-		from subprocess import getstatusoutput as subprocess_getstatusoutput
-
 	import platform
 
 	# Temporarily delete these imports, to ensure that only the
@@ -70,6 +62,7 @@ try:
 			'match_from_list,match_to_list',
 		'portage.dep.dep_check:dep_check,dep_eval,dep_wordreduce,dep_zapdeps',
 		'portage.eclass_cache',
+		'portage.elog',
 		'portage.exception',
 		'portage.getbinpkg',
 		'portage.locks',
@@ -114,6 +107,7 @@ try:
 			'cpv_getkey@getCPFromCPV,endversion_keys,' + \
 			'suffix_value@endversion,pkgcmp,pkgsplit,vercmp,ververify',
 		'portage.xpak',
+		'subprocess',
 		'time',
 	)
 
@@ -294,12 +288,17 @@ class _unicode_module_wrapper(object):
 import os as _os
 _os_overrides = {
 	id(_os.fdopen)        : _os.fdopen,
-	id(_os.mkfifo)        : _os.mkfifo,
 	id(_os.popen)         : _os.popen,
 	id(_os.read)          : _os.read,
 	id(_os.system)        : _os.system,
 }
 
+
+try:
+	_os_overrides[id(_os.mkfifo)] = _os.mkfifo
+except AttributeError:
+	pass # Jython
+
 if hasattr(_os, 'statvfs'):
 	_os_overrides[id(_os.statvfs)] = _os.statvfs
 
@@ -335,6 +334,11 @@ _python_interpreter = os.path.realpath(sys.executable)
 _bin_path = PORTAGE_BIN_PATH
 _pym_path = PORTAGE_PYM_PATH
 
+# Api consumers included in portage should set this to True.
+_internal_warnings = False
+
+_sync_disabled_warnings = False
+
 def _shell_quote(s):
 	"""
 	Quote a string in double-quotes and use backslashes to
@@ -354,8 +358,20 @@ if platform.system() in ('FreeBSD',):
 
 		@classmethod
 		def chflags(cls, path, flags, opts=""):
-			cmd = 'chflags %s %o %s' % (opts, flags, _shell_quote(path))
-			status, output = subprocess_getstatusoutput(cmd)
+			cmd = ['chflags']
+			if opts:
+				cmd.append(opts)
+			cmd.append('%o' % (flags,))
+			cmd.append(path)
+			encoding = _encodings['fs']
+			if sys.hexversion < 0x3000000 or sys.hexversion >= 0x3020000:
+				# Python 3.1 does not support bytes in Popen args.
+				cmd = [_unicode_encode(x, encoding=encoding, errors='strict')
+					for x in cmd]
+			proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+				stderr=subprocess.STDOUT)
+			output = proc.communicate()[0]
+			status = proc.wait()
 			if os.WIFEXITED(status) and os.WEXITSTATUS(status) == os.EX_OK:
 				return
 			# Try to generate an ENOENT error if appropriate.
@@ -368,6 +384,7 @@ if platform.system() in ('FreeBSD',):
 				raise portage.exception.CommandNotFound('chflags')
 			# Now we're not sure exactly why it failed or what
 			# the real errno was, so just report EPERM.
+			output = _unicode_decode(output, encoding=encoding)
 			e = OSError(errno.EPERM, output)
 			e.errno = errno.EPERM
 			e.filename = path
@@ -408,8 +425,8 @@ def abssymlink(symlink, target=None):
 
 _doebuild_manifest_exempt_depend = 0
 
-_testing_eapis = frozenset(["4-python"])
-_deprecated_eapis = frozenset(["4_pre1", "3_pre2", "3_pre1"])
+_testing_eapis = frozenset(["4-python", "4-slot-abi", "5-progress", "5-hdepend"])
+_deprecated_eapis = frozenset(["4_pre1", "3_pre2", "3_pre1", "5_pre1", "5_pre2"])
 
 def _eapi_is_deprecated(eapi):
 	return eapi in _deprecated_eapis
@@ -435,29 +452,25 @@ def eapi_is_supported(eapi):
 		return False
 	return eapi <= portage.const.EAPI
 
-# Generally, it's best not to assume that cache entries for unsupported EAPIs
-# can be validated. However, the current package manager specification does not
-# guarantee that the EAPI can be parsed without sourcing the ebuild, so
-# it's too costly to discard existing cache entries for unsupported EAPIs.
-# Therefore, by default, assume that cache entries for unsupported EAPIs can be
-# validated. If FEATURES=parse-eapi-* is enabled, this assumption is discarded
-# since the EAPI can be determined without the incurring the cost of sourcing
-# the ebuild.
-_validate_cache_for_unsupported_eapis = True
-
-_parse_eapi_ebuild_head_re = re.compile(r'^EAPI=[\'"]?([^\'"#]*)')
-_parse_eapi_ebuild_head_max_lines = 30
+# This pattern is specified by PMS section 7.3.1.
+_pms_eapi_re = re.compile(r"^[ \t]*EAPI=(['\"]?)([A-Za-z0-9+_.-]*)\1[ \t]*([ \t]#.*)?$")
+_comment_or_blank_line = re.compile(r"^\s*(#.*)?$")
 
 def _parse_eapi_ebuild_head(f):
-	count = 0
+	eapi = None
+	eapi_lineno = None
+	lineno = 0
 	for line in f:
-		m = _parse_eapi_ebuild_head_re.match(line)
-		if m is not None:
-			return m.group(1).strip()
-		count += 1
-		if count >= _parse_eapi_ebuild_head_max_lines:
+		lineno += 1
+		m = _comment_or_blank_line.match(line)
+		if m is None:
+			eapi_lineno = lineno
+			m = _pms_eapi_re.match(line)
+			if m is not None:
+				eapi = m.group(2)
 			break
-	return '0'
+
+	return (eapi, eapi_lineno)
 
 def _movefile(src, dest, **kwargs):
 	"""Calls movefile and raises a PortageException if an error occurs."""
@@ -470,14 +483,13 @@ auxdbkeys = (
 	'RESTRICT',  'HOMEPAGE',  'LICENSE',   'DESCRIPTION',
 	'KEYWORDS',  'INHERITED', 'IUSE', 'REQUIRED_USE',
 	'PDEPEND',   'PROVIDE', 'EAPI',
-	'PROPERTIES', 'DEFINED_PHASES', 'UNUSED_05', 'UNUSED_04',
+	'PROPERTIES', 'DEFINED_PHASES', 'HDEPEND', 'UNUSED_04',
 	'UNUSED_03', 'UNUSED_02', 'UNUSED_01',
 )
 auxdbkeylen=len(auxdbkeys)
 
 def portageexit():
-	if data.secpass > 1 and os.environ.get("SANDBOX_ON") != "1":
-		close_portdbapi_caches()
+	close_portdbapi_caches()
 
 class _trees_dict(dict):
 	__slots__ = ('_running_eroot', '_target_eroot',)
@@ -552,11 +564,19 @@ if VERSION == 'HEAD':
 			if VERSION is not self:
 				return VERSION
 			if os.path.isdir(os.path.join(PORTAGE_BASE_PATH, '.git')):
-				status, output = subprocess_getstatusoutput((
-					"cd %s ; git describe --tags || exit $? ; " + \
+				encoding = _encodings['fs']
+				cmd = [BASH_BINARY, "-c", ("cd %s ; git describe --tags || exit $? ; " + \
 					"if [ -n \"`git diff-index --name-only --diff-filter=M HEAD`\" ] ; " + \
 					"then echo modified ; git rev-list --format=%%ct -n 1 HEAD ; fi ; " + \
-					"exit 0") % _shell_quote(PORTAGE_BASE_PATH))
+					"exit 0") % _shell_quote(PORTAGE_BASE_PATH)]
+				if sys.hexversion < 0x3000000 or sys.hexversion >= 0x3020000:
+					# Python 3.1 does not support bytes in Popen args.
+					cmd = [_unicode_encode(x, encoding=encoding, errors='strict')
+						for x in cmd]
+				proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+					stderr=subprocess.STDOUT)
+				output = _unicode_decode(proc.communicate()[0], encoding=encoding)
+				status = proc.wait()
 				if os.WIFEXITED(status) and os.WEXITSTATUS(status) == os.EX_OK:
 					output_lines = output.splitlines()
 					if output_lines:
