@@ -1,18 +1,17 @@
-# Copyright 2010 Gentoo Foundation
+# Copyright 2010-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: dep_check.py 15671 2010-03-02 21:01:10Z zmedico $
 
 __all__ = ['dep_check', 'dep_eval', 'dep_wordreduce', 'dep_zapdeps']
 
 import logging
 
 import portage
-from portage.dep import Atom, dep_opconvert, match_from_list, paren_reduce, \
-	remove_slot, use_reduce
-from portage.exception import InvalidAtom, InvalidDependString, ParseError
+from portage import _unicode_decode
+from portage.dep import Atom, match_from_list, use_reduce
+from portage.exception import InvalidDependString, ParseError
 from portage.localization import _
 from portage.util import writemsg, writemsg_level
-from portage.versions import catpkgsplit, cpv_getkey, pkgcmp
+from portage.versions import vercmp, _pkg_str
 
 def _expand_new_virtuals(mysplit, edebug, mydbapi, mysettings, myroot="/",
 	trees=None, use_mask=None, use_force=None, **kwargs):
@@ -29,22 +28,26 @@ def _expand_new_virtuals(mysplit, edebug, mydbapi, mysettings, myroot="/",
 	newsplit = []
 	mytrees = trees[myroot]
 	portdb = mytrees["porttree"].dbapi
+	pkg_use_enabled = mytrees.get("pkg_use_enabled")
+	# Atoms are stored in the graph as (atom, id(atom)) tuples
+	# since each atom is considered to be a unique entity. For
+	# example, atoms that appear identical may behave differently
+	# in USE matching, depending on their unevaluated form. Also,
+	# specially generated virtual atoms may appear identical while
+	# having different _orig_atom attributes.
 	atom_graph = mytrees.get("atom_graph")
 	parent = mytrees.get("parent")
 	virt_parent = mytrees.get("virt_parent")
 	graph_parent = None
-	eapi = None
 	if parent is not None:
 		if virt_parent is not None:
 			graph_parent = virt_parent
-			eapi = virt_parent[0].metadata['EAPI']
+			parent = virt_parent
 		else:
 			graph_parent = parent
-			eapi = parent.metadata["EAPI"]
 	repoman = not mysettings.local_config
 	if kwargs["use_binaries"]:
 		portdb = trees[myroot]["bintree"].dbapi
-	myvirtuals = mysettings.getvirtuals()
 	pprovideddict = mysettings.pprovideddict
 	myuse = kwargs["myuse"]
 	for x in mysplit:
@@ -58,60 +61,39 @@ def _expand_new_virtuals(mysplit, edebug, mydbapi, mysettings, myroot="/",
 			continue
 
 		if not isinstance(x, Atom):
-			try:
-				x = Atom(x)
-			except InvalidAtom:
-				if portage.dep._dep_check_strict:
-					raise ParseError(
-						_("invalid atom: '%s'") % x)
-				else:
-					# Only real Atom instances are allowed past this point.
-					continue
-			else:
-				if x.blocker and x.blocker.overlap.forbid and \
-					eapi in ("0", "1") and portage.dep._dep_check_strict:
-					raise ParseError(
-						_("invalid atom: '%s'") % (x,))
-				if x.use and eapi in ("0", "1") and \
-					portage.dep._dep_check_strict:
-					raise ParseError(
-						_("invalid atom: '%s'") % (x,))
+			raise ParseError(
+				_("invalid token: '%s'") % x)
 
-		if repoman and x.use and x.use.conditional:
-			evaluated_atom = remove_slot(x)
-			if x.slot:
-				evaluated_atom += ":%s" % x.slot
-			evaluated_atom += str(x.use._eval_qa_conditionals(
-				use_mask, use_force))
-			x = Atom(evaluated_atom)
-
-		if not repoman and \
-			myuse is not None and isinstance(x, Atom) and x.use:
-			if x.use.conditional:
-				x = x.evaluate_conditionals(myuse)
+		if repoman:
+			x = x._eval_qa_conditionals(use_mask, use_force)
 
 		mykey = x.cp
 		if not mykey.startswith("virtual/"):
 			newsplit.append(x)
 			if atom_graph is not None:
-				atom_graph.add(x, graph_parent)
+				atom_graph.add((x, id(x)), graph_parent)
 			continue
-		mychoices = myvirtuals.get(mykey, [])
+
 		if x.blocker:
 			# Virtual blockers are no longer expanded here since
 			# the un-expanded virtual atom is more useful for
 			# maintaining a cache of blocker atoms.
 			newsplit.append(x)
 			if atom_graph is not None:
-				atom_graph.add(x, graph_parent)
+				atom_graph.add((x, id(x)), graph_parent)
 			continue
 
-		if repoman or not hasattr(portdb, 'match_pkgs'):
+		if repoman or not hasattr(portdb, 'match_pkgs') or \
+			pkg_use_enabled is None:
 			if portdb.cp_list(x.cp):
 				newsplit.append(x)
 			else:
 				# TODO: Add PROVIDE check for repoman.
 				a = []
+				myvartree = mytrees.get("vartree")
+				if myvartree is not None:
+					mysettings._populate_treeVirtuals_if_needed(myvartree)
+				mychoices = mysettings.getvirtuals().get(mykey, [])
 				for y in mychoices:
 					a.append(Atom(x.replace(x.cp, y.cp, 1)))
 				if not a:
@@ -133,6 +115,14 @@ def _expand_new_virtuals(mysplit, edebug, mydbapi, mysettings, myroot="/",
 			# only use new-style matches
 			if pkg.cp.startswith("virtual/"):
 				pkgs.append(pkg)
+
+		mychoices = []
+		if not pkgs and not portdb.cp_list(x.cp):
+			myvartree = mytrees.get("vartree")
+			if myvartree is not None:
+				mysettings._populate_treeVirtuals_if_needed(myvartree)
+			mychoices = mysettings.getvirtuals().get(mykey, [])
+
 		if not (pkgs or mychoices):
 			# This one couldn't be expanded as a new-style virtual.  Old-style
 			# virtuals have already been expanded by dep_virtual, so this one
@@ -141,21 +131,38 @@ def _expand_new_virtuals(mysplit, edebug, mydbapi, mysettings, myroot="/",
 			# dependency that needs to be satisfied.
 			newsplit.append(x)
 			if atom_graph is not None:
-				atom_graph.add(x, graph_parent)
+				atom_graph.add((x, id(x)), graph_parent)
 			continue
 
 		a = []
 		for pkg in pkgs:
 			virt_atom = '=' + pkg.cpv
-			if x.use:
-				virt_atom += str(x.use)
-			virt_atom = Atom(virt_atom)
+			if x.unevaluated_atom.use:
+				virt_atom += str(x.unevaluated_atom.use)
+				virt_atom = Atom(virt_atom)
+				if parent is None:
+					if myuse is None:
+						virt_atom = virt_atom.evaluate_conditionals(
+							mysettings.get("PORTAGE_USE", "").split())
+					else:
+						virt_atom = virt_atom.evaluate_conditionals(myuse)
+				else:
+					virt_atom = virt_atom.evaluate_conditionals(
+						pkg_use_enabled(parent))
+			else:
+				virt_atom = Atom(virt_atom)
+
+			# Allow the depgraph to map this atom back to the
+			# original, in order to avoid distortion in places
+			# like display or conflict resolution code.
+			virt_atom.__dict__['_orig_atom'] = x
+
 			# According to GLEP 37, RDEPEND is the only dependency
 			# type that is valid for new-style virtuals. Repoman
 			# should enforce this.
 			depstring = pkg.metadata['RDEPEND']
 			pkg_kwargs = kwargs.copy()
-			pkg_kwargs["myuse"] = pkg.use.enabled
+			pkg_kwargs["myuse"] = pkg_use_enabled(pkg)
 			if edebug:
 				writemsg_level(_("Virtual Parent:      %s\n") \
 					% (pkg,), noiselevel=-1, level=logging.DEBUG)
@@ -163,7 +170,7 @@ def _expand_new_virtuals(mysplit, edebug, mydbapi, mysettings, myroot="/",
 					% (depstring,), noiselevel=-1, level=logging.DEBUG)
 
 			# Set EAPI used for validation in dep_check() recursion.
-			mytrees["virt_parent"] = (pkg, virt_atom)
+			mytrees["virt_parent"] = pkg
 
 			try:
 				mycheck = dep_check(depstring, mydbapi, mysettings,
@@ -176,14 +183,16 @@ def _expand_new_virtuals(mysplit, edebug, mydbapi, mysettings, myroot="/",
 					del mytrees["virt_parent"]
 
 			if not mycheck[0]:
-				raise ParseError(
-					"%s: %s '%s'" % (y[0], mycheck[1], depstring))
+				raise ParseError(_unicode_decode("%s: %s '%s'") % \
+					(pkg, mycheck[1], depstring))
 
 			# pull in the new-style virtual
 			mycheck[1].append(virt_atom)
 			a.append(mycheck[1])
 			if atom_graph is not None:
-				atom_graph.add(virt_atom, graph_parent)
+				virt_atom_node = (virt_atom, id(virt_atom))
+				atom_graph.add(virt_atom_node, graph_parent)
+				atom_graph.add(pkg, virt_atom_node)
 		# Plain old-style virtuals.  New-style virtuals are preferred.
 		if not pkgs:
 				for y in mychoices:
@@ -195,7 +204,8 @@ def _expand_new_virtuals(mysplit, edebug, mydbapi, mysettings, myroot="/",
 						portdb.aux_get(matches[-1], ['PROVIDE'])[0].split():
 						a.append(new_atom)
 						if atom_graph is not None:
-							atom_graph.add(new_atom, graph_parent)
+							atom_graph.add((new_atom, id(new_atom)),
+								graph_parent)
 
 		if not a and mychoices:
 			# Check for a virtual package.provided match.
@@ -205,12 +215,12 @@ def _expand_new_virtuals(mysplit, edebug, mydbapi, mysettings, myroot="/",
 					pprovideddict.get(new_atom.cp, [])):
 					a.append(new_atom)
 					if atom_graph is not None:
-						atom_graph.add(new_atom, graph_parent)
+						atom_graph.add((new_atom, id(new_atom)), graph_parent)
 
 		if not a:
 			newsplit.append(x)
 			if atom_graph is not None:
-				atom_graph.add(x, graph_parent)
+				atom_graph.add((x, id(x)), graph_parent)
 		elif len(a) == 1:
 			newsplit.append(a[0])
 		else:
@@ -282,6 +292,8 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 	unsat_use_in_graph = []
 	unsat_use_installed = []
 	unsat_use_non_installed = []
+	other_installed = []
+	other_installed_some = []
 	other = []
 
 	# unsat_use_* must come after preferred_non_installed
@@ -294,6 +306,8 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 		unsat_use_in_graph,
 		unsat_use_installed,
 		unsat_use_non_installed,
+		other_installed,
+		other_installed_some,
 		other,
 	)
 
@@ -301,6 +315,7 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 	parent   = trees[myroot].get("parent")
 	priority = trees[myroot].get("priority")
 	graph_db = trees[myroot].get("graph_db")
+	graph    = trees[myroot].get("graph")
 	vardb = None
 	if "vartree" in trees[myroot]:
 		vardb = trees[myroot]["vartree"].dbapi
@@ -335,8 +350,8 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 			avail_pkg = mydbapi.match(atom.without_use)
 			if avail_pkg:
 				avail_pkg = avail_pkg[-1] # highest (ascending order)
-				avail_slot = Atom("%s:%s" % (atom.cp,
-					mydbapi.aux_get(avail_pkg, ["SLOT"])[0]))
+				avail_pkg = mydbapi._pkg_str(avail_pkg, atom.repo)
+				avail_slot = Atom("%s:%s" % (atom.cp, avail_pkg.slot))
 			if not avail_pkg:
 				all_available = False
 				all_use_satisfied = False
@@ -351,16 +366,14 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 					avail_pkg_use = avail_pkg_use[-1]
 					if avail_pkg_use != avail_pkg:
 						avail_pkg = avail_pkg_use
-						avail_slot = Atom("%s:%s" % (atom.cp,
-							mydbapi.aux_get(avail_pkg, ["SLOT"])[0]))
+					avail_pkg = mydbapi._pkg_str(avail_pkg, atom.repo)
+					avail_slot = Atom("%s:%s" % (atom.cp, avail_pkg.slot))
 
 			slot_map[avail_slot] = avail_pkg
-			pkg_cp = cpv_getkey(avail_pkg)
-			highest_cpv = cp_map.get(pkg_cp)
+			highest_cpv = cp_map.get(avail_pkg.cp)
 			if highest_cpv is None or \
-				pkgcmp(catpkgsplit(avail_pkg)[1:],
-				catpkgsplit(highest_cpv)[1:]) > 0:
-				cp_map[pkg_cp] = avail_pkg
+				vercmp(avail_pkg.version, highest_cpv.version) > 0:
+				cp_map[avail_pkg.cp] = avail_pkg
 
 		this_choice = (atoms, slot_map, cp_map, all_available)
 		if all_available:
@@ -401,15 +414,22 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 				all_in_graph = True
 				for slot_atom in slot_map:
 					# New-style virtuals have zero cost to install.
-					if not graph_db.match(slot_atom) and \
-						not slot_atom.startswith("virtual/"):
+					if slot_atom.startswith("virtual/"):
+						continue
+					# We check if the matched package has actually been
+					# added to the digraph, in order to distinguish between
+					# those packages and installed packages that may need
+					# to be uninstalled in order to resolve blockers.
+					graph_matches = graph_db.match_pkgs(slot_atom)
+					if not graph_matches or graph_matches[-1] not in graph:
 						all_in_graph = False
 						break
 				circular_atom = None
 				if all_in_graph:
 					if parent is None or priority is None:
 						pass
-					elif priority.buildtime:
+					elif priority.buildtime and \
+						not (priority.satisfied or priority.optional):
 						# Check if the atom would result in a direct circular
 						# dependency and try to avoid that if it seems likely
 						# to be unresolvable. This is only relevant for
@@ -449,7 +469,21 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 						else:
 							unsat_use_non_installed.append(this_choice)
 		else:
-			other.append(this_choice)
+			all_installed = True
+			some_installed = False
+			for atom in atoms:
+				if not atom.blocker:
+					if vardb.match(atom):
+						some_installed = True
+					else:
+						all_installed = False
+
+			if all_installed:
+				other_installed.append(this_choice)
+			elif some_installed:
+				other_installed_some.append(this_choice)
+			else:
+				other.append(this_choice)
 
 	# Prefer choices which contain upgrades to higher slots. This helps
 	# for deps such as || ( foo:1 foo:2 ), where we want to prefer the
@@ -477,8 +511,7 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 				for cp in intersecting_cps:
 					version_1 = cp_map_1[cp]
 					version_2 = cp_map_2[cp]
-					difference = pkgcmp(catpkgsplit(version_1)[1:],
-						catpkgsplit(version_2)[1:])
+					difference = vercmp(version_1.version, version_2.version)
 					if difference != 0:
 						if difference > 0:
 							has_upgrade = True
@@ -501,8 +534,12 @@ def dep_zapdeps(unreduced, reduced, myroot, use_binaries=0, trees=None):
 	assert(False) # This point should not be reachable
 
 def dep_check(depstring, mydbapi, mysettings, use="yes", mode=None, myuse=None,
-	use_cache=1, use_binaries=0, myroot="/", trees=None):
-	"""Takes a depend string and parses the condition."""
+	use_cache=1, use_binaries=0, myroot=None, trees=None):
+	"""
+	Takes a depend string, parses it, and selects atoms.
+	The myroot parameter is unused (use mysettings['EROOT'] instead).
+	"""
+	myroot = mysettings['EROOT']
 	edebug = mysettings.get("PORTAGE_DEBUG", None) == "1"
 	#check_config_instance(mysettings)
 	if trees is None:
@@ -525,36 +562,50 @@ def dep_check(depstring, mydbapi, mysettings, use="yes", mode=None, myuse=None,
 		# WE ALSO CANNOT USE SETTINGS
 		myusesplit=[]
 
-	#convert parenthesis to sublists
-	try:
-		mysplit = paren_reduce(depstring)
-	except InvalidDependString as e:
-		return [0, str(e)]
-
 	mymasks = set()
 	useforce = set()
-	useforce.add(mysettings["ARCH"])
 	if use == "all":
-		# This masking/forcing is only for repoman.  In other cases, relevant
-		# masking/forcing should have already been applied via
-		# config.regenerate().  Also, binary or installed packages may have
-		# been built with flags that are now masked, and it would be
-		# inconsistent to mask them now.  Additionally, myuse may consist of
-		# flags from a parent package that is being merged to a $ROOT that is
-		# different from the one that mysettings represents.
+		# This is only for repoman, in order to constrain the use_reduce
+		# matchall behavior to account for profile use.mask/force. The
+		# ARCH/archlist code here may be redundant, since the profile
+		# really should be handling ARCH masking/forcing itself.
 		mymasks.update(mysettings.usemask)
 		mymasks.update(mysettings.archlist())
 		mymasks.discard(mysettings["ARCH"])
+		useforce.add(mysettings["ARCH"])
 		useforce.update(mysettings.useforce)
 		useforce.difference_update(mymasks)
-	try:
-		mysplit = use_reduce(mysplit, uselist=myusesplit,
-			masklist=mymasks, matchall=(use=="all"), excludeall=useforce)
-	except InvalidDependString as e:
-		return [0, str(e)]
 
-	# Do the || conversions
-	mysplit = dep_opconvert(mysplit)
+	# eapi code borrowed from _expand_new_virtuals()
+	mytrees = trees[myroot]
+	parent = mytrees.get("parent")
+	virt_parent = mytrees.get("virt_parent")
+	current_parent = None
+	eapi = None
+	if parent is not None:
+		if virt_parent is not None:
+			current_parent = virt_parent
+		else:
+			current_parent = parent
+
+	if current_parent is not None:
+		# Don't pass the eapi argument to use_reduce() for installed packages
+		# since previous validation will have already marked them as invalid
+		# when necessary and now we're more interested in evaluating
+		# dependencies so that things like --depclean work as well as possible
+		# in spite of partial invalidity.
+		if not current_parent.installed:
+			eapi = current_parent.metadata['EAPI']
+
+	if isinstance(depstring, list):
+		mysplit = depstring
+	else:
+		try:
+			mysplit = use_reduce(depstring, uselist=myusesplit,
+			masklist=mymasks, matchall=(use=="all"), excludeall=useforce,
+			opconvert=True, token_class=Atom, eapi=eapi)
+		except InvalidDependString as e:
+			return [0, _unicode_decode("%s") % (e,)]
 
 	if mysplit == []:
 		#dependencies were reduced to nothing
@@ -568,7 +619,7 @@ def dep_check(depstring, mydbapi, mysettings, use="yes", mode=None, myuse=None,
 			use_force=useforce, use_mask=mymasks, use_cache=use_cache,
 			use_binaries=use_binaries, myroot=myroot, trees=trees)
 	except ParseError as e:
-		return [0, str(e)]
+		return [0, _unicode_decode("%s") % (e,)]
 
 	mysplit2=mysplit[:]
 	mysplit2=dep_wordreduce(mysplit2,mysettings,mydbapi,mode,use_cache=use_cache)
@@ -579,15 +630,8 @@ def dep_check(depstring, mydbapi, mysettings, use="yes", mode=None, myuse=None,
 	writemsg("mysplit:  %s\n" % (mysplit), 1)
 	writemsg("mysplit2: %s\n" % (mysplit2), 1)
 
-	try:
-		selected_atoms = dep_zapdeps(mysplit, mysplit2, myroot,
-			use_binaries=use_binaries, trees=trees)
-	except InvalidAtom as e:
-		if portage.dep._dep_check_strict:
-			raise # This shouldn't happen.
-		# dbapi.match() failed due to an invalid atom in
-		# the dependencies of an installed package.
-		return [0, _("Invalid atom: '%s'") % (e,)]
+	selected_atoms = dep_zapdeps(mysplit, mysplit2, myroot,
+		use_binaries=use_binaries, trees=trees)
 
 	return [1, selected_atoms]
 
