@@ -1,7 +1,6 @@
 # getbinpkg.py -- Portage binary-package helper functions
-# Copyright 2003-2004 Gentoo Foundation
+# Copyright 2003-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: getbinpkg.py 15246 2010-01-29 18:47:51Z zmedico $
 
 from portage.output import colorize
 from portage.cache.mappings import slot_dict_class
@@ -9,13 +8,19 @@ from portage.localization import _
 import portage
 from portage import os
 from portage import _encodings
+from portage import _unicode_decode
 from portage import _unicode_encode
+from portage.package.ebuild.fetch import _hide_url_passwd
+from _emerge.Package import _all_metadata_keys
 
 import sys
 import socket
 import time
 import tempfile
 import base64
+import warnings
+
+_all_errors = [NotImplementedError, ValueError, socket.error]
 
 try:
 	from html.parser import HTMLParser as html_parser_HTMLParser
@@ -24,7 +29,7 @@ except ImportError:
 
 try:
 	from urllib.parse import unquote as urllib_parse_unquote
-except:
+except ImportError:
 	from urllib2 import unquote as urllib_parse_unquote
 
 try:
@@ -36,30 +41,47 @@ try:
 	import ftplib
 except ImportError as e:
 	sys.stderr.write(colorize("BAD","!!! CANNOT IMPORT FTPLIB: ")+str(e)+"\n")
+else:
+	_all_errors.extend(ftplib.all_errors)
 
 try:
 	try:
 		from http.client import HTTPConnection as http_client_HTTPConnection
-		from http.client import HTTPSConnection as http_client_HTTPSConnection
 		from http.client import BadStatusLine as http_client_BadStatusLine
 		from http.client import ResponseNotReady as http_client_ResponseNotReady
+		from http.client import error as http_client_error
 	except ImportError:
 		from httplib import HTTPConnection as http_client_HTTPConnection
-		from httplib import HTTPSConnection as http_client_HTTPSConnection
 		from httplib import BadStatusLine as http_client_BadStatusLine
 		from httplib import ResponseNotReady as http_client_ResponseNotReady
+		from httplib import error as http_client_error
 except ImportError as e:
 	sys.stderr.write(colorize("BAD","!!! CANNOT IMPORT HTTP.CLIENT: ")+str(e)+"\n")
+else:
+	_all_errors.append(http_client_error)
+
+_all_errors = tuple(_all_errors)
 
 if sys.hexversion >= 0x3000000:
 	long = int
 
 def make_metadata_dict(data):
+
+	warnings.warn("portage.getbinpkg.make_metadata_dict() is deprecated",
+		DeprecationWarning, stacklevel=2)
+
 	myid,myglob = data
 	
 	mydict = {}
-	for x in portage.xpak.getindex_mem(myid):
-		mydict[x] = portage.xpak.getitem(data,x)
+	for k_bytes in portage.xpak.getindex_mem(myid):
+		k = _unicode_decode(k_bytes,
+			encoding=_encodings['repo.content'], errors='replace')
+		if k not in _all_metadata_keys and \
+			k != "CATEGORY":
+			continue
+		v = _unicode_decode(portage.xpak.getitem(data, k_bytes),
+			encoding=_encodings['repo.content'], errors='replace')
+		mydict[k] = v
 
 	return mydict
 
@@ -67,6 +89,10 @@ class ParseLinks(html_parser_HTMLParser):
 	"""Parser class that overrides HTMLParser to grab all anchors from an html
 	page and provide suffix and prefix limitors"""
 	def __init__(self):
+
+		warnings.warn("portage.getbinpkg.ParseLinks is deprecated",
+			DeprecationWarning, stacklevel=2)
+
 		self.PL_anchors = []
 		html_parser_HTMLParser.__init__(self)
 
@@ -105,6 +131,9 @@ def create_conn(baseurl,conn=None):
 	optional connection. If connection is already active, it is passed on.
 	baseurl is reduced to address and is returned in tuple (conn,address)"""
 
+	warnings.warn("portage.getbinpkg.create_conn() is deprecated",
+		DeprecationWarning, stacklevel=2)
+
 	parts = baseurl.split("://",1)
 	if len(parts) != 2:
 		raise ValueError(_("Provided URI does not "
@@ -142,16 +171,33 @@ def create_conn(baseurl,conn=None):
 	http_headers = {}
 	http_params = {}
 	if username and password:
+		try:
+			encodebytes = base64.encodebytes
+		except AttributeError:
+			# Python 2
+			encodebytes = base64.encodestring
 		http_headers = {
-			"Authorization": "Basic %s" %
-			  base64.encodestring("%s:%s" % (username, password)).replace(
-			    "\012",
-			    ""
+			b"Authorization": "Basic %s" % \
+			encodebytes(_unicode_encode("%s:%s" % (username, password))).replace(
+			    b"\012",
+			    b""
 			  ),
 		}
 
 	if not conn:
 		if protocol == "https":
+			# Use local import since https typically isn't needed, and
+			# this way we can usually avoid triggering the global scope
+			# http.client ImportError handler (like during stage1 -> stage2
+			# builds where USE=ssl is disabled for python).
+			try:
+				try:
+					from http.client import HTTPSConnection as http_client_HTTPSConnection
+				except ImportError:
+					from httplib import HTTPSConnection as http_client_HTTPSConnection
+			except ImportError:
+				raise NotImplementedError(
+					_("python must have ssl enabled for https support"))
 			conn = http_client_HTTPSConnection(host)
 		elif protocol == "http":
 			conn = http_client_HTTPConnection(host)
@@ -187,6 +233,10 @@ def create_conn(baseurl,conn=None):
 def make_ftp_request(conn, address, rest=None, dest=None):
 	"""(conn,address,rest) --- uses the conn object to request the data
 	from address and issuing a rest if it is passed."""
+
+	warnings.warn("portage.getbinpkg.make_ftp_request() is deprecated",
+		DeprecationWarning, stacklevel=2)
+
 	try:
 	
 		if dest:
@@ -236,13 +286,16 @@ def make_http_request(conn, address, params={}, headers={}, dest=None):
 	the data from address, performing Location forwarding and using the
 	optional params and headers."""
 
+	warnings.warn("portage.getbinpkg.make_http_request() is deprecated",
+		DeprecationWarning, stacklevel=2)
+
 	rc = 0
 	response = None
 	while (rc == 0) or (rc == 301) or (rc == 302):
 		try:
 			if (rc != 0):
 				conn,ignore,ignore,ignore,ignore = create_conn(address)
-			conn.request("GET", address, params, headers)
+			conn.request("GET", address, body=None, headers=headers)
 		except SystemExit as e:
 			raise
 		except Exception as e:
@@ -278,6 +331,10 @@ def make_http_request(conn, address, params={}, headers={}, dest=None):
 
 
 def match_in_array(array, prefix="", suffix="", match_both=1, allow_overlap=0):
+
+	warnings.warn("portage.getbinpkg.match_in_array() is deprecated",
+		DeprecationWarning, stacklevel=2)
+
 	myarray = []
 	
 	if not (prefix and suffix):
@@ -318,6 +375,9 @@ def dir_get_list(baseurl,conn=None):
 	URI should be in the form <proto>://<site>[:port]<path>
 	Connection is used for persistent connection instances."""
 
+	warnings.warn("portage.getbinpkg.dir_get_list() is deprecated",
+		DeprecationWarning, stacklevel=2)
+
 	if not conn:
 		keepconnection = 0
 	else:
@@ -335,7 +395,7 @@ def dir_get_list(baseurl,conn=None):
 		
 		if page:
 			parser = ParseLinks()
-			parser.feed(page)
+			parser.feed(_unicode_decode(page))
 			del page
 			listing = parser.get_anchors()
 		else:
@@ -366,6 +426,9 @@ def file_get_metadata(baseurl,conn=None, chunk_size=3000):
 	URI should be in the form <proto>://<site>[:port]<path>
 	Connection is used for persistent connection instances."""
 
+	warnings.warn("portage.getbinpkg.file_get_metadata() is deprecated",
+		DeprecationWarning, stacklevel=2)
+
 	if not conn:
 		keepconnection = 0
 	else:
@@ -412,17 +475,23 @@ def file_get_metadata(baseurl,conn=None, chunk_size=3000):
 	return myid
 
 
-def file_get(baseurl,dest,conn=None,fcmd=None):
+def file_get(baseurl,dest,conn=None,fcmd=None,filename=None):
 	"""(baseurl,dest,fcmd=) -- Takes a base url to connect to and read from.
 	URI should be in the form <proto>://[user[:pass]@]<site>[:port]<path>"""
 
 	if not fcmd:
+
+		warnings.warn("Use of portage.getbinpkg.file_get() without the fcmd "
+			"parameter is deprecated", DeprecationWarning, stacklevel=2)
+
 		return file_get_lib(baseurl,dest,conn)
+	if not filename:
+		filename = os.path.basename(baseurl)
 
 	variables = {
 		"DISTDIR": dest,
 		"URI":     baseurl,
-		"FILE":    os.path.basename(baseurl)
+		"FILE":    filename
 	}
 
 	from portage.util import varexpand
@@ -430,10 +499,12 @@ def file_get(baseurl,dest,conn=None,fcmd=None):
 	myfetch = portage.util.shlex_split(fcmd)
 	myfetch = [varexpand(x, mydict=variables) for x in myfetch]
 	fd_pipes= {
-		0:sys.stdin.fileno(),
-		1:sys.stdout.fileno(),
-		2:sys.stdout.fileno()
+		0:sys.__stdin__.fileno(),
+		1:sys.__stdout__.fileno(),
+		2:sys.__stdout__.fileno()
 	}
+	sys.__stdout__.flush()
+	sys.__stderr__.flush()
 	retval = spawn(myfetch, env=os.environ.copy(), fd_pipes=fd_pipes)
 	if retval != os.EX_OK:
 		sys.stderr.write(_("Fetcher exited with a failure condition.\n"))
@@ -445,6 +516,9 @@ def file_get_lib(baseurl,dest,conn=None):
 	URI should be in the form <proto>://<site>[:port]<path>
 	Connection is used for persistent connection instances."""
 
+	warnings.warn("portage.getbinpkg.file_get_lib() is deprecated",
+		DeprecationWarning, stacklevel=2)
+
 	if not conn:
 		keepconnection = 0
 	else:
@@ -488,6 +562,10 @@ def file_get_lib(baseurl,dest,conn=None):
 def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=1, makepickle=None):
 	"""(baseurl,conn,chunk_size,verbose) -- 
 	"""
+
+	warnings.warn("portage.getbinpkg.dir_get_metadata() is deprecated",
+		DeprecationWarning, stacklevel=2)
+
 	if not conn:
 		keepconnection = 0
 	else:
@@ -501,7 +579,7 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 
 	try:
 		conn, protocol, address, params, headers = create_conn(baseurl, conn)
-	except socket.error as e:
+	except _all_errors as e:
 		# ftplib.FTP(host) can raise errors like this:
 		#   socket.error: (111, 'Connection refused')
 		sys.stderr.write("!!! %s\n" % (e,))
@@ -521,7 +599,9 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 		out.write(_("Loaded metadata pickle.\n"))
 		out.flush()
 		metadatafile.close()
-	except (IOError, OSError, EOFError, ValueError, pickle.UnpicklingError):
+	except (SystemExit, KeyboardInterrupt):
+		raise
+	except Exception:
 		metadata = {}
 	if baseurl not in metadata:
 		metadata[baseurl]={}
@@ -543,7 +623,8 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 	try:
 		filelist = dir_get_list(baseurl, conn)
 	except portage.exception.PortageException as e:
-		sys.stderr.write(_("!!! Error connecting to '%s'.\n") % baseurl)
+		sys.stderr.write(_("!!! Error connecting to '%s'.\n") %
+			_hide_url_passwd(baseurl))
 		sys.stderr.write("!!! %s\n" % str(e))
 		del e
 		return metadata[baseurl]["data"]
