@@ -1,9 +1,9 @@
-# Copyright 2003-2010 Gentoo Foundation
+# Copyright 2003-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from __future__ import absolute_import
 
-import codecs
+import io
 import sys
 try:
 	from urllib.request import urlopen as urllib_request_urlopen
@@ -17,10 +17,11 @@ from portage import os
 from portage import _encodings
 from portage import _unicode_decode
 from portage import _unicode_encode
-from portage.versions import pkgsplit, catpkgsplit, pkgcmp, best
+from portage.versions import pkgsplit, vercmp, best
 from portage.util import grabfile
 from portage.const import CACHE_PATH
 from portage.localization import _
+from portage.dep import _slot_separator
 
 # Note: the space for rgt and rlt is important !!
 # FIXME: use slot deps instead, requires GLSA format versioning
@@ -242,7 +243,7 @@ def makeAtom(pkgname, versionNode):
 		pass
 	else:
 		if slot and slot != "*":
-			rValue += ":" + slot
+			rValue += _slot_separator + slot
 	return str(rValue)
 
 def makeVersion(versionNode):
@@ -264,7 +265,7 @@ def makeVersion(versionNode):
 		pass
 	else:
 		if slot and slot != "*":
-			rValue += ":" + slot
+			rValue += _slot_separator + slot
 	return rValue
 
 def match(atom, dbapi, match_type="default"):
@@ -371,17 +372,14 @@ def getMinUpgrade(vulnerableList, unaffectedList, portdbapi, vardbapi, minimize=
 	for u in unaffectedList:
 		mylist = match(u, portdbapi, match_type="match-all")
 		for c in mylist:
-			c_pv = catpkgsplit(c)
-			i_pv = catpkgsplit(best(v_installed))
-			if pkgcmp(c_pv[1:], i_pv[1:]) > 0 \
+			i = best(v_installed)
+			if vercmp(c.version, i.version) > 0 \
 					and (rValue == None \
 						or not match("="+rValue, portdbapi) \
-						or (minimize ^ (pkgcmp(c_pv[1:], catpkgsplit(rValue)[1:]) > 0)) \
+						or (minimize ^ (vercmp(c.version, rValue.version) > 0)) \
 							and match("="+c, portdbapi)) \
-					and portdbapi.aux_get(c, ["SLOT"]) == vardbapi.aux_get(best(v_installed), ["SLOT"]):
-				rValue = c_pv[0]+"/"+c_pv[1]+"-"+c_pv[2]
-				if c_pv[3] != "r0":		# we don't like -r0 for display
-					rValue += "-"+c_pv[3]
+					and portdbapi._pkg_str(c, None).slot == vardbapi._pkg_str(best(v_installed), None).slot:
+				rValue = c
 	return rValue
 
 def format_date(datestr):
@@ -475,7 +473,13 @@ class Glsa:
 			myurl = "file://"+self.nr
 		else:
 			myurl = repository + "glsa-%s.xml" % str(self.nr)
-		self.parse(urllib_request_urlopen(myurl))
+
+		f = urllib_request_urlopen(myurl)
+		try:
+			self.parse(f)
+		finally:
+			f.close()
+
 		return None
 
 	def parse(self, myfile):
@@ -487,7 +491,7 @@ class Glsa:
 		@type	myfile: String
 		@param	myfile: Filename to grab the XML data from
 		@rtype:		None
-		@returns:	None
+		@return:	None
 		"""
 		self.DOM = xml.dom.minidom.parse(myfile)
 		if not self.DOM.doctype:
@@ -633,7 +637,7 @@ class Glsa:
 		architectures.
 		
 		@rtype:		Boolean
-		@returns:	True if the system is affected, False if not
+		@return:	True if the system is affected, False if not
 		"""
 		rValue = False
 		for k in self.packages:
@@ -653,7 +657,7 @@ class Glsa:
 		GLSA was already applied.
 		
 		@rtype:		Boolean
-		@returns:	True if the GLSA was applied, False if not
+		@return:	True if the GLSA was applied, False if not
 		"""
 		return (self.nr in get_applied_glsas(self.config))
 
@@ -664,15 +668,15 @@ class Glsa:
 		applied or on explicit user request.
 
 		@rtype:		None
-		@returns:	None
+		@return:	None
 		"""
 		if not self.isApplied():
-			checkfile = codecs.open(
+			checkfile = io.open(
 				_unicode_encode(os.path.join(self.config["EROOT"],
 				CACHE_PATH, "glsa"),
 				encoding=_encodings['fs'], errors='strict'), 
 				mode='a+', encoding=_encodings['content'], errors='strict')
-			checkfile.write(self.nr+"\n")
+			checkfile.write(_unicode_decode(self.nr + "\n"))
 			checkfile.close()
 		return None
 	
