@@ -1,17 +1,15 @@
-# Copyright 2010 Gentoo Foundation
+# Copyright 2010-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: fetch.py 15718 2010-03-02 21:37:48Z zmedico $
 
 from __future__ import print_function
 
 __all__ = ['fetch']
 
-import codecs
 import errno
+import io
 import logging
 import random
 import re
-import shutil
 import stat
 import sys
 import tempfile
@@ -20,18 +18,26 @@ import tempfile
 import portage
 portage.proxy.lazyimport.lazyimport(globals(),
 	'portage.package.ebuild.config:check_config_instance,config',
+	'portage.package.ebuild.doebuild:doebuild_environment,' + \
+		'_doebuild_spawn',
+	'portage.package.ebuild.prepare_build_dirs:prepare_build_dirs',
 )
 
-from portage import doebuild_environment, OrderedDict, os, prepare_build_dirs, selinux, _encodings, _shell_quote, _unicode_encode
-from portage.checksum import perform_md5, verify_all
-from portage.const import BASH_BINARY, CUSTOM_MIRRORS_FILE, EBUILD_SH_BINARY, GLOBAL_CONFIG_PATH
+from portage import OrderedDict, os, selinux, shutil, _encodings, \
+	_shell_quote, _unicode_encode
+from portage.checksum import (hashfunc_map, perform_md5, verify_all,
+	_filter_unaccelarated_hashes, _hash_filter, _apply_hash_filter)
+from portage.const import BASH_BINARY, CUSTOM_MIRRORS_FILE, \
+	GLOBAL_CONFIG_PATH
 from portage.data import portage_gid, portage_uid, secpass, userpriv_groups
-from portage.exception import FileNotFound, OperationNotPermitted, PermissionDenied, PortageException, TryAgain
+from portage.exception import FileNotFound, OperationNotPermitted, \
+	PortageException, TryAgain
 from portage.localization import _
 from portage.locks import lockfile, unlockfile
-from portage.manifest import Manifest
 from portage.output import colorize, EOutput
-from portage.util import apply_recursive_permissions, apply_secpass_permissions, ensure_dirs, grabdict, shlex_split, varexpand, writemsg, writemsg_level, writemsg_stdout
+from portage.util import apply_recursive_permissions, \
+	apply_secpass_permissions, ensure_dirs, grabdict, shlex_split, \
+	varexpand, writemsg, writemsg_level, writemsg_stdout
 from portage.process import spawn
 
 _userpriv_spawn_kwargs = (
@@ -41,6 +47,9 @@ _userpriv_spawn_kwargs = (
 	("umask",  0o02),
 )
 
+def _hide_url_passwd(url):
+	return re.sub(r'//(.+):.+@(.+)', r'//\1:*password*@\2', url)
+
 def _spawn_fetch(settings, args, **kwargs):
 	"""
 	Spawn a process with appropriate settings for fetching, including
@@ -55,13 +64,14 @@ def _spawn_fetch(settings, args, **kwargs):
 	if "fd_pipes" not in kwargs:
 
 		kwargs["fd_pipes"] = {
-			0 : sys.stdin.fileno(),
-			1 : sys.stdout.fileno(),
-			2 : sys.stdout.fileno(),
+			0 : sys.__stdin__.fileno(),
+			1 : sys.__stdout__.fileno(),
+			2 : sys.__stdout__.fileno(),
 		}
 
 	if "userfetch" in settings.features and \
-		os.getuid() == 0 and portage_gid and portage_uid:
+		os.getuid() == 0 and portage_gid and portage_uid and \
+		hasattr(os, "setgroups"):
 		kwargs.update(_userpriv_spawn_kwargs)
 
 	spawn_func = spawn
@@ -74,12 +84,23 @@ def _spawn_fetch(settings, args, **kwargs):
 		if args[0] != BASH_BINARY:
 			args = [BASH_BINARY, "-c", "exec \"$@\"", args[0]] + args
 
-	rval = spawn_func(args, env=settings.environ(), **kwargs)
+	# Ensure that EBUILD_PHASE is set to fetch, so that config.environ()
+	# does not filter the calling environment (which may contain needed
+	# proxy variables, as in bug #315421).
+	phase_backup = settings.get('EBUILD_PHASE')
+	settings['EBUILD_PHASE'] = 'fetch'
+	try:
+		rval = spawn_func(args, env=settings.environ(), **kwargs)
+	finally:
+		if phase_backup is None:
+			settings.pop('EBUILD_PHASE', None)
+		else:
+			settings['EBUILD_PHASE'] = phase_backup
 
 	return rval
 
 _userpriv_test_write_file_cache = {}
-_userpriv_test_write_cmd_script = "touch %(file_path)s 2>/dev/null ; rval=$? ; " + \
+_userpriv_test_write_cmd_script = ">> %(file_path)s 2>/dev/null ; rval=$? ; " + \
 	"rm -f  %(file_path)s ; exit $rval"
 
 def _userpriv_test_write_file(settings, file_path):
@@ -164,7 +185,7 @@ def _check_digests(filename, digests, show_errors=1):
 		return False
 	return True
 
-def _check_distfile(filename, digests, eout, show_errors=1):
+def _check_distfile(filename, digests, eout, show_errors=1, hash_filter=None):
 	"""
 	@return a tuple of (match, stat_obj) where match is True if filename
 	matches all given digests (if any) and stat_obj is a stat result, or
@@ -190,6 +211,9 @@ def _check_distfile(filename, digests, eout, show_errors=1):
 			# Zero-byte distfiles are always invalid.
 			return (False, st)
 	else:
+		digests = _filter_unaccelarated_hashes(digests)
+		if hash_filter is not None:
+			digests = _apply_hash_filter(digests, hash_filter)
 		if _check_digests(filename, digests, show_errors=show_errors):
 			eout.ebegin("%s %s ;-)" % (os.path.basename(filename),
 				" ".join(sorted(digests))))
@@ -212,7 +236,9 @@ _size_suffix_map = {
 	'Y' : 80,
 }
 
-def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",use_locks=1, try_mirrors=1):
+def fetch(myuris, mysettings, listonly=0, fetchonly=0,
+	locks_in_subdir=".locks", use_locks=1, try_mirrors=1, digests=None,
+	allow_missing_digests=True):
 	"fetch files.  Will use digest file if available."
 
 	if not myuris:
@@ -225,8 +251,8 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 	userpriv = secpass >= 2 and "userpriv" in features
 
 	# 'nomirror' is bad/negative logic. You Restrict mirroring, not no-mirroring.
-	if "mirror" in restrict or \
-	   "nomirror" in restrict:
+	restrict_mirror = "mirror" in restrict or "nomirror" in restrict
+	if restrict_mirror:
 		if ("mirror" in features) and ("lmirror" not in features):
 			# lmirror should allow you to bypass mirror restrictions.
 			# XXX: This is not a good thing, and is temporary at best.
@@ -324,22 +350,29 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 	if "local" in custommirrors:
 		mymirrors += custommirrors["local"]
 
-	if "nomirror" in restrict or \
-	   "mirror" in restrict:
+	if restrict_mirror:
 		# We don't add any mirrors.
 		pass
 	else:
 		if try_mirrors:
 			mymirrors += [x.rstrip("/") for x in mysettings["GENTOO_MIRRORS"].split() if x]
 
+	hash_filter = _hash_filter(mysettings.get("PORTAGE_CHECKSUM_FILTER", ""))
+	if hash_filter.transparent:
+		hash_filter = None
 	skip_manifest = mysettings.get("EBUILD_SKIP_MANIFEST") == "1"
+	if skip_manifest:
+		allow_missing_digests = True
 	pkgdir = mysettings.get("O")
-	if not (pkgdir is None or skip_manifest):
-		mydigests = Manifest(
+	if digests is None and not (pkgdir is None or skip_manifest):
+		mydigests = mysettings.repositories.get_repo_for_location(
+			os.path.dirname(os.path.dirname(pkgdir))).load_manifest(
 			pkgdir, mysettings["DISTDIR"]).getTypeDigests("DIST")
-	else:
+	elif digests is None or skip_manifest:
 		# no digests because fetch was not called for a specific package
 		mydigests = {}
+	else:
+		mydigests = digests
 
 	ro_distdirs = [x for x in \
 		shlex_split(mysettings.get("PORTAGE_RO_DISTDIRS", "")) \
@@ -352,6 +385,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 			del mymirrors[x]
 
 	restrict_fetch = "fetch" in restrict
+	force_mirror = "force-mirror" in features and not restrict_mirror
 	custom_local_mirrors = custommirrors.get("local", [])
 	if restrict_fetch:
 		# With fetch restriction, a normal uri may only be fetched from
@@ -373,7 +407,6 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 			file_uri_tuples.append((os.path.basename(myuri), myuri))
 
 	filedict = OrderedDict()
-	primaryuri_indexes={}
 	primaryuri_dict = {}
 	thirdpartymirror_uris = {}
 	for myfile, myuri in file_uri_tuples:
@@ -395,10 +428,9 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 
 				# now try the official mirrors
 				if mirrorname in thirdpartymirrors:
-					random.shuffle(thirdpartymirrors[mirrorname])
-
 					uris = [locmirr.rstrip("/") + "/" + path \
 						for locmirr in thirdpartymirrors[mirrorname]]
+					random.shuffle(uris)
 					filedict[myfile].extend(uris)
 					thirdpartymirror_uris.setdefault(myfile, []).extend(uris)
 
@@ -408,29 +440,33 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 				writemsg(_("Invalid mirror definition in SRC_URI:\n"), noiselevel=-1)
 				writemsg("  %s\n" % (myuri), noiselevel=-1)
 		else:
-			if restrict_fetch:
+			if restrict_fetch or force_mirror:
 				# Only fetch from specific mirrors is allowed.
 				continue
-			if "primaryuri" in restrict:
-				# Use the source site first.
-				if myfile in primaryuri_indexes:
-					primaryuri_indexes[myfile] += 1
-				else:
-					primaryuri_indexes[myfile] = 0
-				filedict[myfile].insert(primaryuri_indexes[myfile], myuri)
-			else:
-				filedict[myfile].append(myuri)
 			primaryuris = primaryuri_dict.get(myfile)
 			if primaryuris is None:
 				primaryuris = []
 				primaryuri_dict[myfile] = primaryuris
 			primaryuris.append(myuri)
 
+	# Order primaryuri_dict values to match that in SRC_URI.
+	for uris in primaryuri_dict.values():
+		uris.reverse()
+
 	# Prefer thirdpartymirrors over normal mirrors in cases when
 	# the file does not yet exist on the normal mirrors.
 	for myfile, uris in thirdpartymirror_uris.items():
 		primaryuri_dict.setdefault(myfile, []).extend(uris)
 
+	# Now merge primaryuri values into filedict (includes mirrors
+	# explicitly referenced in SRC_URI).
+	if "primaryuri" in restrict:
+		for myfile, uris in filedict.items():
+			filedict[myfile] = primaryuri_dict.get(myfile, []) + uris
+	else:
+		for myfile in filedict:
+			filedict[myfile] += primaryuri_dict.get(myfile, [])
+
 	can_fetch=True
 
 	if listonly:
@@ -438,7 +474,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 
 	if can_fetch and not fetch_to_ro:
 		global _userpriv_test_write_file_cache
-		dirmode  = 0o2070
+		dirmode  = 0o070
 		filemode =   0o60
 		modemask =    0o2
 		dir_gid = portage_gid
@@ -449,8 +485,6 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 			# when inside fakeroot.
 			dir_gid = 0
 		distdir_dirs = [""]
-		if "distlocks" in features:
-			distdir_dirs.append(".locks")
 		try:
 			
 			for x in distdir_dirs:
@@ -497,14 +531,6 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 			noiselevel=-1)
 		can_fetch = False
 
-	if can_fetch and use_locks and locks_in_subdir:
-			distlocks_subdir = os.path.join(mysettings["DISTDIR"], locks_in_subdir)
-			if not os.access(distlocks_subdir, os.W_OK):
-				writemsg(_("!!! No write access to write to %s.  Aborting.\n") % distlocks_subdir,
-					noiselevel=-1)
-				return 0
-			del distlocks_subdir
-
 	distdir_writable = can_fetch and not fetch_to_ro
 	failed_files = set()
 	restrict_fetch_msg = False
@@ -519,6 +545,32 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 		fetched = 0
 
 		orig_digests = mydigests.get(myfile, {})
+
+		if not (allow_missing_digests or listonly):
+			verifiable_hash_types = set(orig_digests).intersection(hashfunc_map)
+			verifiable_hash_types.discard("size")
+			if not verifiable_hash_types:
+				expected = set(hashfunc_map)
+				expected.discard("size")
+				expected = " ".join(sorted(expected))
+				got = set(orig_digests)
+				got.discard("size")
+				got = " ".join(sorted(got))
+				reason = (_("Insufficient data for checksum verification"),
+					got, expected)
+				writemsg(_("!!! Fetched file: %s VERIFY FAILED!\n") % myfile,
+					noiselevel=-1)
+				writemsg(_("!!! Reason: %s\n") % reason[0],
+					noiselevel=-1)
+				writemsg(_("!!! Got:      %s\n!!! Expected: %s\n") % \
+					(reason[1], reason[2]), noiselevel=-1)
+
+				if fetchonly:
+					failed_files.add(myfile)
+					continue
+				else:
+					return 0
+
 		size = orig_digests.get("size")
 		if size == 0:
 			# Zero-byte distfiles are always invalid, so discard their digests.
@@ -540,8 +592,17 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 		else:
 			# check if there is enough space in DISTDIR to completely store myfile
 			# overestimate the filesize so we aren't bitten by FS overhead
+			vfs_stat = None
 			if size is not None and hasattr(os, "statvfs"):
-				vfs_stat = os.statvfs(mysettings["DISTDIR"])
+				try:
+					vfs_stat = os.statvfs(mysettings["DISTDIR"])
+				except OSError as e:
+					writemsg_level("!!! statvfs('%s'): %s\n" %
+						(mysettings["DISTDIR"], e),
+						noiselevel=-1, level=logging.ERROR)
+					del e
+
+			if vfs_stat is not None:
 				try:
 					mysize = os.stat(myfile_path).st_size
 				except OSError as e:
@@ -563,18 +624,6 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 					elif userfetch:
 						has_space = False
 
-			if not has_space:
-				writemsg(_("!!! Insufficient space to store %s in %s\n") % \
-					(myfile, mysettings["DISTDIR"]), noiselevel=-1)
-
-				if has_space_superuser:
-					writemsg(_("!!! Insufficient privileges to use "
-						"remaining space.\n"), noiselevel=-1)
-					if userfetch:
-						writemsg(_("!!! You may set FEATURES=\"-userfetch\""
-							" in /etc/make.conf in order to fetch with\n"
-							"!!! superuser privileges.\n"), noiselevel=-1)
-
 			if distdir_writable and use_locks:
 
 				lock_kwargs = {}
@@ -595,9 +644,12 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 				eout = EOutput()
 				eout.quiet = mysettings.get("PORTAGE_QUIET") == "1"
 				match, mystat = _check_distfile(
-					myfile_path, pruned_digests, eout)
+					myfile_path, pruned_digests, eout, hash_filter=hash_filter)
 				if match:
-					if distdir_writable:
+					# Skip permission adjustment for symlinks, since we don't
+					# want to modify anything outside of the primary DISTDIR,
+					# and symlinks typically point to PORTAGE_RO_DISTDIRS.
+					if distdir_writable and not os.path.islink(myfile_path):
 						try:
 							apply_secpass_permissions(myfile_path,
 								gid=portage_gid, mode=0o664, mask=0o2,
@@ -664,7 +716,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 					for x in ro_distdirs:
 						filename = os.path.join(x, myfile)
 						match, mystat = _check_distfile(
-							filename, pruned_digests, eout)
+							filename, pruned_digests, eout, hash_filter=hash_filter)
 						if match:
 							readonly_file = filename
 							break
@@ -678,6 +730,20 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 						os.symlink(readonly_file, myfile_path)
 						continue
 
+				# this message is shown only after we know that
+				# the file is not already fetched
+				if not has_space:
+					writemsg(_("!!! Insufficient space to store %s in %s\n") % \
+						(myfile, mysettings["DISTDIR"]), noiselevel=-1)
+
+					if has_space_superuser:
+						writemsg(_("!!! Insufficient privileges to use "
+							"remaining space.\n"), noiselevel=-1)
+						if userfetch:
+							writemsg(_("!!! You may set FEATURES=\"-userfetch\""
+								" in /etc/make.conf in order to fetch with\n"
+								"!!! superuser privileges.\n"), noiselevel=-1)
+
 				if fsmirrors and not os.path.exists(myfile_path) and has_space:
 					for mydir in fsmirrors:
 						mirror_file = os.path.join(mydir, myfile)
@@ -697,14 +763,18 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 						raise
 					del e
 				else:
-					try:
-						apply_secpass_permissions(
-							myfile_path, gid=portage_gid, mode=0o664, mask=0o2,
-							stat_cached=mystat)
-					except PortageException as e:
-						if not os.access(myfile_path, os.R_OK):
-							writemsg(_("!!! Failed to adjust permissions:"
-								" %s\n") % str(e), noiselevel=-1)
+					# Skip permission adjustment for symlinks, since we don't
+					# want to modify anything outside of the primary DISTDIR,
+					# and symlinks typically point to PORTAGE_RO_DISTDIRS.
+					if not os.path.islink(myfile_path):
+						try:
+							apply_secpass_permissions(myfile_path,
+								gid=portage_gid, mode=0o664, mask=0o2,
+								stat_cached=mystat)
+						except PortageException as e:
+							if not os.access(myfile_path, os.R_OK):
+								writemsg(_("!!! Failed to adjust permissions:"
+									" %s\n") % (e,), noiselevel=-1)
 
 					# If the file is empty then it's obviously invalid. Remove
 					# the empty file and try to download if possible.
@@ -732,8 +802,10 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 							eout.eend(0)
 							continue
 						else:
-							verified_ok, reason = verify_all(
-								myfile_path, mydigests[myfile])
+							digests = _filter_unaccelarated_hashes(mydigests[myfile])
+							if hash_filter is not None:
+								digests = _apply_hash_filter(digests, hash_filter)
+							verified_ok, reason = verify_all(myfile_path, digests)
 							if not verified_ok:
 								writemsg(_("!!! Previously fetched"
 									" file: '%s'\n") % myfile, noiselevel=-1)
@@ -755,7 +827,6 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 								eout = EOutput()
 								eout.quiet = \
 									mysettings.get("PORTAGE_QUIET", None) == "1"
-								digests = mydigests.get(myfile)
 								if digests:
 									digests = list(digests)
 									digests.sort()
@@ -782,6 +853,11 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 				# allow different fetchcommands per protocol
 				protocol = loc[0:loc.find("://")]
 
+				global_config_path = GLOBAL_CONFIG_PATH
+				if mysettings['EPREFIX']:
+					global_config_path = os.path.join(mysettings['EPREFIX'],
+							GLOBAL_CONFIG_PATH.lstrip(os.sep))
+
 				missing_file_param = False
 				fetchcommand_var = "FETCHCOMMAND_" + protocol.upper()
 				fetchcommand = mysettings.get(fetchcommand_var)
@@ -792,7 +868,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 						writemsg_level(
 							_("!!! %s is unset. It should "
 							"have been defined in\n!!! %s/make.globals.\n") \
-							% (fetchcommand_var, GLOBAL_CONFIG_PATH),
+							% (fetchcommand_var, global_config_path),
 							level=logging.ERROR, noiselevel=-1)
 						return 0
 				if "${FILE}" not in fetchcommand:
@@ -811,7 +887,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 						writemsg_level(
 							_("!!! %s is unset. It should "
 							"have been defined in\n!!! %s/make.globals.\n") \
-							% (resumecommand_var, GLOBAL_CONFIG_PATH),
+							% (resumecommand_var, global_config_path),
 							level=logging.ERROR, noiselevel=-1)
 						return 0
 				if "${FILE}" not in resumecommand:
@@ -886,7 +962,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 						locfetch=fetchcommand
 						command_var = fetchcommand_var
 					writemsg_stdout(_(">>> Downloading '%s'\n") % \
-						re.sub(r'//(.+):.+@(.+)/',r'//\1:*password*@\2/', loc))
+						_hide_url_passwd(loc))
 					variables = {
 						"DISTDIR": mysettings["DISTDIR"],
 						"URI":     loc,
@@ -965,18 +1041,19 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 								# Fetch failed... Try the next one... Kill 404 files though.
 								if (mystat[stat.ST_SIZE]<100000) and (len(myfile)>4) and not ((myfile[-5:]==".html") or (myfile[-4:]==".htm")):
 									html404=re.compile("<title>.*(not found|404).*</title>",re.I|re.M)
-									if html404.search(codecs.open(
+									with io.open(
 										_unicode_encode(myfile_path,
 										encoding=_encodings['fs'], errors='strict'),
 										mode='r', encoding=_encodings['content'], errors='replace'
-										).read()):
-										try:
-											os.unlink(mysettings["DISTDIR"]+"/"+myfile)
-											writemsg(_(">>> Deleting invalid distfile. (Improper 404 redirect from server.)\n"))
-											fetched = 0
-											continue
-										except (IOError, OSError):
-											pass
+										) as f:
+										if html404.search(f.read()):
+											try:
+												os.unlink(mysettings["DISTDIR"]+"/"+myfile)
+												writemsg(_(">>> Deleting invalid distfile. (Improper 404 redirect from server.)\n"))
+												fetched = 0
+												continue
+											except (IOError, OSError):
+												pass
 								fetched = 1
 								continue
 							if True:
@@ -984,9 +1061,11 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 								# file NOW, for those users who don't have a stable/continuous
 								# net connection. This way we have a chance to try to download
 								# from another mirror...
-								verified_ok,reason = verify_all(mysettings["DISTDIR"]+"/"+myfile, mydigests[myfile])
+								digests = _filter_unaccelarated_hashes(mydigests[myfile])
+								if hash_filter is not None:
+									digests = _apply_hash_filter(digests, hash_filter)
+								verified_ok, reason = verify_all(myfile_path, digests)
 								if not verified_ok:
-									print(reason)
 									writemsg(_("!!! Fetched file: %s VERIFY FAILED!\n") % myfile,
 										noiselevel=-1)
 									writemsg(_("!!! Reason: %s\n") % reason[0],
@@ -1019,7 +1098,6 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 								else:
 									eout = EOutput()
 									eout.quiet = mysettings.get("PORTAGE_QUIET", None) == "1"
-									digests = mydigests.get(myfile)
 									if digests:
 										eout.ebegin("%s %s ;-)" % \
 											(myfile, " ".join(sorted(digests))))
@@ -1036,6 +1114,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 		finally:
 			if use_locks and file_lock:
 				unlockfile(file_lock)
+				file_lock = None
 
 		if listonly:
 			writemsg_stdout("\n", noiselevel=-1)
@@ -1051,61 +1130,6 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 					(mysettings["CATEGORY"], mysettings["PF"])
 				writemsg_level(msg,
 					level=logging.ERROR, noiselevel=-1)
-				have_builddir = "PORTAGE_BUILDDIR" in mysettings and \
-					os.path.isdir(mysettings["PORTAGE_BUILDDIR"])
-
-				global_tmpdir = mysettings["PORTAGE_TMPDIR"]
-				private_tmpdir = None
-				if not parallel_fetchonly and not have_builddir:
-					# When called by digestgen(), it's normal that
-					# PORTAGE_BUILDDIR doesn't exist. It's helpful
-					# to show the pkg_nofetch output though, so go
-					# ahead and create a temporary PORTAGE_BUILDDIR.
-					# Use a temporary config instance to avoid altering
-					# the state of the one that's been passed in.
-					mysettings = config(clone=mysettings)
-					try:
-						private_tmpdir = tempfile.mkdtemp("", "._portage_fetch_.",
-							global_tmpdir)
-					except OSError as e:
-						if e.errno != PermissionDenied.errno:
-							raise
-						raise PermissionDenied(global_tmpdir)
-					mysettings["PORTAGE_TMPDIR"] = private_tmpdir
-					mysettings.backup_changes("PORTAGE_TMPDIR")
-					debug = mysettings.get("PORTAGE_DEBUG") == "1"
-					doebuild_environment(mysettings["EBUILD"], "fetch",
-						mysettings["ROOT"], mysettings, debug, 1, None)
-					prepare_build_dirs(mysettings["ROOT"], mysettings, 0)
-					have_builddir = True
-
-				if not parallel_fetchonly and have_builddir:
-					# To spawn pkg_nofetch requires PORTAGE_BUILDDIR for
-					# ensuring sane $PWD (bug #239560) and storing elog
-					# messages. Therefore, calling code needs to ensure that
-					# PORTAGE_BUILDDIR is already clean and locked here.
-
-					# All the pkg_nofetch goes to stderr since it's considered
-					# to be an error message.
-					fd_pipes = {
-						0 : sys.stdin.fileno(),
-						1 : sys.stderr.fileno(),
-						2 : sys.stderr.fileno(),
-					}
-
-					ebuild_phase = mysettings.get("EBUILD_PHASE")
-					try:
-						mysettings["EBUILD_PHASE"] = "nofetch"
-						spawn(_shell_quote(EBUILD_SH_BINARY) + \
-							" nofetch", mysettings, fd_pipes=fd_pipes)
-					finally:
-						if ebuild_phase is None:
-							mysettings.pop("EBUILD_PHASE", None)
-						else:
-							mysettings["EBUILD_PHASE"] = ebuild_phase
-						if private_tmpdir is not None:
-							shutil.rmtree(private_tmpdir)
-
 			elif restrict_fetch:
 				pass
 			elif listonly:
@@ -1118,6 +1142,7 @@ def fetch(myuris, mysettings, listonly=0, fetchonly=0, locks_in_subdir=".locks",
 					noiselevel=-1)
 
 			if listonly:
+				failed_files.add(myfile)
 				continue
 			elif fetchonly:
 				failed_files.add(myfile)
