@@ -1,21 +1,21 @@
 # portage.py -- core Portage functionality
-# Copyright 1998-2009 Gentoo Foundation
+# Copyright 1998-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: process.py 14364 2009-09-21 20:29:30Z zmedico $
 
 
 import atexit
+import errno
+import platform
 import signal
 import sys
 import traceback
 
 from portage import os
 from portage import _encodings
-from portage import _unicode_decode
 from portage import _unicode_encode
 import portage
 portage.proxy.lazyimport.lazyimport(globals(),
-	'portage.util:dump_traceback',
+	'portage.util:dump_traceback,writemsg',
 )
 
 from portage.const import BASH_BINARY, SANDBOX_BINARY, FAKEROOT_BINARY
@@ -34,6 +34,18 @@ if os.path.isdir("/proc/%i/fd" % os.getpid()):
 	def get_open_fds():
 		return (int(fd) for fd in os.listdir("/proc/%i/fd" % os.getpid()) \
 			if fd.isdigit())
+
+	if platform.python_implementation() == 'PyPy':
+		# EAGAIN observed with PyPy 1.8.
+		_get_open_fds = get_open_fds
+		def get_open_fds():
+			try:
+				return _get_open_fds()
+			except OSError as e:
+				if e.errno != errno.EAGAIN:
+					raise
+				return range(max_fd_limit)
+
 else:
 	def get_open_fds():
 		return range(max_fd_limit)
@@ -137,7 +149,10 @@ def cleanup():
 	while spawned_pids:
 		pid = spawned_pids.pop()
 		try:
-			if os.waitpid(pid, os.WNOHANG) == (0, 0):
+			# With waitpid and WNOHANG, only check the
+			# first element of the tuple since the second
+			# element may vary (bug #337465).
+			if os.waitpid(pid, os.WNOHANG)[0] == 0:
 				os.kill(pid, signal.SIGTERM)
 				os.waitpid(pid, 0)
 		except OSError:
@@ -208,12 +223,12 @@ def spawn(mycommand, env={}, opt_name=None, fd_pipes=None, returnpid=False,
 			raise CommandNotFound(mycommand[0])
 
 	# If we haven't been told what file descriptors to use
-	# default to propogating our stdin, stdout and stderr.
+	# default to propagating our stdin, stdout and stderr.
 	if fd_pipes is None:
 		fd_pipes = {
-			0:sys.stdin.fileno(),
-			1:sys.stdout.fileno(),
-			2:sys.stderr.fileno(),
+			0:sys.__stdin__.fileno(),
+			1:sys.__stdout__.fileno(),
+			2:sys.__stderr__.fileno(),
 		}
 
 	# mypids will hold the pids of all processes created.
@@ -241,21 +256,38 @@ def spawn(mycommand, env={}, opt_name=None, fd_pipes=None, returnpid=False,
 		fd_pipes[1] = pw
 		fd_pipes[2] = pw
 
-	pid = os.fork()
+	parent_pid = os.getpid()
+	pid = None
+	try:
+		pid = os.fork()
 
-	if not pid:
-		try:
-			_exec(binary, mycommand, opt_name, fd_pipes,
-			      env, gid, groups, uid, umask, pre_exec)
-		except Exception as e:
-			# We need to catch _any_ exception so that it doesn't
-			# propogate out of this function and cause exiting
-			# with anything other than os._exit()
-			sys.stderr.write("%s:\n   %s\n" % (e, " ".join(mycommand)))
-			traceback.print_exc()
-			sys.stderr.flush()
+		if pid == 0:
+			try:
+				_exec(binary, mycommand, opt_name, fd_pipes,
+					env, gid, groups, uid, umask, pre_exec)
+			except SystemExit:
+				raise
+			except Exception as e:
+				# We need to catch _any_ exception so that it doesn't
+				# propagate out of this function and cause exiting
+				# with anything other than os._exit()
+				writemsg("%s:\n   %s\n" % (e, " ".join(mycommand)),
+					noiselevel=-1)
+				traceback.print_exc()
+				sys.stderr.flush()
+
+	finally:
+		if pid == 0 or (pid is None and os.getpid() != parent_pid):
+			# Call os._exit() from a finally block in order
+			# to suppress any finally blocks from earlier
+			# in the call stack (see bug #345289). This
+			# finally block has to be setup before the fork
+			# in order to avoid a race condition.
 			os._exit(1)
 
+	if not isinstance(pid, int):
+		raise AssertionError("fork returned non-integer: %s" % (repr(pid),))
+
 	# Add the pid to our local and the global pid lists.
 	mypids.append(pid)
 	spawned_pids.append(pid)
@@ -289,7 +321,10 @@ def spawn(mycommand, env={}, opt_name=None, fd_pipes=None, returnpid=False,
 			# If it failed, kill off anything else that
 			# isn't dead yet.
 			for pid in mypids:
-				if os.waitpid(pid, os.WNOHANG) == (0,0):
+				# With waitpid and WNOHANG, only check the
+				# first element of the tuple since the second
+				# element may vary (bug #337465).
+				if os.waitpid(pid, os.WNOHANG)[0] == 0:
 					os.kill(pid, signal.SIGTERM)
 					os.waitpid(pid, 0)
 				spawned_pids.remove(pid)
@@ -331,36 +366,36 @@ def _exec(binary, mycommand, opt_name, fd_pipes, env, gid, groups, uid, umask,
 	@param pre_exec: A function to be called with no arguments just prior to the exec call.
 	@type pre_exec: callable
 	@rtype: None
-	@returns: Never returns (calls os.execve)
+	@return: Never returns (calls os.execve)
 	"""
 	
 	# If the process we're creating hasn't been given a name
 	# assign it the name of the executable.
 	if not opt_name:
-		opt_name = os.path.basename(binary)
+		if binary is portage._python_interpreter:
+			# NOTE: PyPy 1.7 will die due to "libary path not found" if argv[0]
+			# does not contain the full path of the binary.
+			opt_name = binary
+		else:
+			opt_name = os.path.basename(binary)
 
 	# Set up the command's argument list.
 	myargs = [opt_name]
 	myargs.extend(mycommand[1:])
 
-	# Set up the command's pipes.
-	my_fds = {}
-	# To protect from cases where direct assignment could
-	# clobber needed fds ({1:2, 2:1}) we first dupe the fds
-	# into unused fds.
-	for fd in fd_pipes:
-		my_fds[fd] = os.dup(fd_pipes[fd])
-	# Then assign them to what they should be.
-	for fd in my_fds:
-		os.dup2(my_fds[fd], fd)
-	# Then close _all_ fds that haven't been explictly
-	# requested to be kept open.
-	for fd in get_open_fds():
-		if fd not in my_fds:
-			try:
-				os.close(fd)
-			except OSError:
-				pass
+	# Use default signal handlers in order to avoid problems
+	# killing subprocesses as reported in bug #353239.
+	signal.signal(signal.SIGINT, signal.SIG_DFL)
+	signal.signal(signal.SIGTERM, signal.SIG_DFL)
+
+	# Quiet killing of subprocesses by SIGPIPE (see bug #309001).
+	signal.signal(signal.SIGPIPE, signal.SIG_DFL)
+
+	# Avoid issues triggered by inheritance of SIGQUIT handler from
+	# the parent process (see bug #289486).
+	signal.signal(signal.SIGQUIT, signal.SIG_DFL)
+
+	_setup_pipes(fd_pipes)
 
 	# Set requested process permissions.
 	if gid:
@@ -377,6 +412,40 @@ def _exec(binary, mycommand, opt_name, fd_pipes, env, gid, groups, uid, umask,
 	# And switch to the new process.
 	os.execve(binary, myargs, env)
 
+def _setup_pipes(fd_pipes, close_fds=True):
+	"""Setup pipes for a forked process.
+
+	WARNING: When not followed by exec, the close_fds behavior
+	can trigger interference from destructors that close file
+	descriptors. This interference happens when the garbage
+	collector intermittently executes such destructors after their
+	corresponding file descriptors have been re-used, leading
+	to intermittent "[Errno 9] Bad file descriptor" exceptions in
+	forked processes. This problem has been observed with PyPy 1.8,
+	and also with CPython under some circumstances (as triggered
+	by xmpppy in bug #374335). In order to close a safe subset of
+	file descriptors, see portage.locks._close_fds().
+	"""
+	my_fds = {}
+	# To protect from cases where direct assignment could
+	# clobber needed fds ({1:2, 2:1}) we first dupe the fds
+	# into unused fds.
+	for fd in fd_pipes:
+		my_fds[fd] = os.dup(fd_pipes[fd])
+	# Then assign them to what they should be.
+	for fd in my_fds:
+		os.dup2(my_fds[fd], fd)
+
+	if close_fds:
+		# Then close _all_ fds that haven't been explicitly
+		# requested to be kept open.
+		for fd in get_open_fds():
+			if fd not in my_fds:
+				try:
+					os.close(fd)
+				except OSError:
+					pass
+
 def find_binary(binary):
 	"""
 	Given a binary name, find the binary in PATH
@@ -384,7 +453,7 @@ def find_binary(binary):
 	@param binary: Name of the binary to find
 	@type string
 	@rtype: None or string
-	@returns: full path to binary or None if the binary could not be located.
+	@return: full path to binary or None if the binary could not be located.
 	"""
 	for path in os.environ.get("PATH", "").split(":"):
 		filename = "%s/%s" % (path, binary)
